VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 1  'Persistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCell"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' The different available controls
Public Enum EditControls
    Edit_TextBox
    Edit_ComboBox
End Enum

' The type of edit-box to utilize
Public EditControl As EditControls
Attribute EditControl.VB_VarDescription = "The method of editing to utilize."

' Used in conjunction with the combobox control
Public EditData As Variant
Attribute EditData.VB_VarDescription = "Used in conjuncton wth the combobox-style of edititing, in which it should contain an array of all items to show."

' The text to draw
Public Text As String
Attribute Text.VB_VarDescription = "Returns/sets the text to draw."

' Key used to identify the column
Public Name As String
Attribute Name.VB_VarUserMemId = 0
Attribute Name.VB_VarDescription = "Returns/sets the name/key of the column. Ignored if the object is a cell."

' Different pictures to draw
Public Pictures As New Collection
Attribute Pictures.VB_VarDescription = "Returns the collection of pictures that is assosiated with this cell/column."

' Different styles
Public StyleNormal As clsStyle
Attribute StyleNormal.VB_VarDescription = "Returns/sets the default style of this cell/column."
Public StyleSelected As clsStyle
Attribute StyleSelected.VB_VarDescription = "Returns/sets the style to use when the cell is selected."
Public StylePressed As clsStyle
Attribute StylePressed.VB_VarDescription = "Returns/sets the style to use when the column is pressed."
Public StyleHover As clsStyle
Attribute StyleHover.VB_VarDescription = "Returns/sets the style to use when the cell/column is hovering."

' Whether or not the cell is currently being hovered or pressed
Public Hovering As Boolean
Attribute Hovering.VB_VarDescription = "Returns whether or not the cursor is currently hovering above the cell."
Public Pressing As Boolean
Attribute Pressing.VB_VarDescription = "Returns whether or not the user is currently clicking this cell/column."

' The width and height (ignored when the cell is not a column header)
Public Width As Long
Public Height As Long
Attribute Height.VB_VarDescription = "The height of the cell. Ignored if this is a column."

' Visible locations (where the cell is on the screen)
Public x As Long
Attribute x.VB_VarDescription = "Returns the last drawn X-position of this cell/column."
Public y As Long
Attribute y.VB_VarDescription = "Returns the last drawn Y-position of this cell/column."

' Used to relocate the object relative to the above location
Public OffsetX As Long
Attribute OffsetX.VB_VarDescription = "Returns/sets the amount of pixels in the X-axis the cell is moved from its proper location."
Public OffsetY As Long
Attribute OffsetY.VB_VarDescription = "Returns/sets the amount of pixels in the Y-axis the cell is moved from its proper location."

' Of all registered icons in the current style, only the following is to be drawed
Public Icon As String
Attribute Icon.VB_VarDescription = "Returns/sets the icon picture in the respected style to draw."

' Extra data free to be used by the program (its preservation cannot be
' promised in non-column headers)
Public Tag As Variant
Attribute Tag.VB_VarDescription = "Stores any extra data needed for your program."

' Information regarding the positioning of this cell
Public Column As Long
Attribute Column.VB_VarDescription = "Returns the index to the column this cell belongs to."
Public Row As Long ' (ignored in the case of column headers, obviously)
Attribute Row.VB_VarDescription = "Returns the row index of the column/cell. If above zero, the object is a cell. Otherwise, it's a column."

' Whether or not to draw this cell (or column)
Public Visible As Boolean
Attribute Visible.VB_VarDescription = "Returns/sets whether or not this cell/column is visible or not."

' Our parent user control
Public Parent As usrListView
Attribute Parent.VB_VarDescription = "Returns the parent user control of this cell/column."

' References to the different styles
Private pStyles As Collection

Public Sub InitializeClass(refParent As usrListView)
Attribute InitializeClass.VB_Description = "Internal procedure used to reference the external styles in the user control."

    On Error Resume Next

    ' Set the parent
    Set Parent = refParent

    ' Now, onto the initialization of the styles. As these are no more than references, we'll have
    ' to load (using name references) the real styles in the parent control.
    Set StyleNormal = Parent.Styles(pStyles("StyleNormal"))
    Set StyleSelected = Parent.Styles(pStyles("StyleSelected"))
    Set StylePressed = Parent.Styles(pStyles("StylePressed"))
    Set StyleHover = Parent.Styles(pStyles("StyleHover"))

End Sub

Private Sub Class_ReadProperties(PropBag As PropertyBag)

    On Error Resume Next
    Dim bEmpty() As Byte

    ' Initialize a new collection
    Set pStyles = New Collection

    ' Use the current property bag
    With PropBag
    
        ' Save all properties that is relevant for a column header
        Text = .ReadProperty("Text", "")
        Width = .ReadProperty("Width", 0)
        Height = .ReadProperty("Height", 0)
        Tag = .ReadProperty("Tag", "")
        Column = .ReadProperty("Column", 0)
        Row = .ReadProperty("Row", 0)
        OffsetX = .ReadProperty("OffsetX", 0)
        OffsetY = .ReadProperty("OffsetY", 0)
        Visible = .ReadProperty("Visible", True)
        
        ' Save the style indexes
        pStyles.Add .ReadProperty("StyleNormal", ""), "StyleNormal"
        pStyles.Add .ReadProperty("StyleSelected", ""), "StyleSelected"
        pStyles.Add .ReadProperty("StylePressed", ""), "StylePressed"
        pStyles.Add .ReadProperty("StyleHover", ""), "StyleHover"
        
        ' Loads all pictures (background, icons, ect.)
        Set Pictures = RetrieveCollection(.ReadProperty("Pictures", bEmpty))
        
    End With

End Sub

Private Sub Class_WriteProperties(PropBag As PropertyBag)

    On Error Resume Next

    ' Use the property bag
    With PropBag

        ' Save all properties
        .WriteProperty "Text", Text, ""
        .WriteProperty "Width", Width, 0
        .WriteProperty "Height", Height, 0
        .WriteProperty "OffsetX", OffsetX, 0
        .WriteProperty "OffsetY", OffsetY, 0
        .WriteProperty "Tag", Tag, ""
        .WriteProperty "Column", Column, 0
        .WriteProperty "Row", Row, 0
        .WriteProperty "Visible", Visible, True
        .WriteProperty "StyleNormal", StyleNormal.Name, ""
        .WriteProperty "StyleSelected", StyleSelected.Name, ""
        .WriteProperty "StylePressed", StylePressed.Name, ""
        .WriteProperty "StyleHover", StyleHover.Name, ""
        .WriteProperty "Pictures", ConvertCollection(Pictures).Contents

    End With

End Sub
