VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 1  'Persistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsPicture"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Location to draw this image
Public X As Single
Attribute X.VB_VarDescription = "Returns/sets the X-position to draw this image."
Public Y As Single
Attribute Y.VB_VarDescription = "Returns/sets the Y-position to draw this image."

' The size of the resulting picture
Public Width As Long
Attribute Width.VB_VarDescription = "Returns/sets the width of this picture."
Public Height As Long
Attribute Height.VB_VarDescription = "The height of the picture."

' Where to start paining from in the source picture (clipping)
Public Clipping As New clsRect
Attribute Clipping.VB_VarDescription = "The location in the source picture to draw."

' The opcode to use
Public Opcode As RasterOpConstants
Attribute Opcode.VB_VarDescription = "Returns/sets the opcode to use when drawing this picture."

' Used to repeatedly draw the picture in either direction
Public RepeatX As Boolean
Attribute RepeatX.VB_VarDescription = "Returns/sets whether or not to draw this picture repeatedly from left to write."
Public RepeatY As Boolean
Attribute RepeatY.VB_VarDescription = "Returns/sets whether or not to draw this picture repeatedly from the top to the bottom."

' Whether or not this picture is a so-called "icon"
Public Icon As Boolean
Attribute Icon.VB_VarDescription = "Whether or not this picture is an icon. Used in conjunction with the Icon-property of a cell."

' The name (tag) of this picutre
Public Name As String
Attribute Name.VB_VarUserMemId = 0
Attribute Name.VB_VarDescription = "Returns/sets the name/key of this picture."

' The image to use
Private pPicture As StdPicture

Public Sub DrawPicture(refSurface As Object, ByVal addX As Long, ByVal addY As Long, ByVal maxWidth As Long, ByVal maxHeight As Long)
Attribute DrawPicture.VB_Description = "Called to draw the current picture upon a picture box."

    ' Start drawing the picture
    refSurface.PaintPicture Picture, addX + X, addY + Y, Width, Height, Clipping.Left, _
     Clipping.Top, Clipping.Right - Clipping.Left, Clipping.Bottom - Clipping.Top, Opcode

    ' See if we have to continue on drawing (X-axis)
    If RepeatX Then
        If addX + X + Width < maxWidth Then
            DrawPicture refSurface, addX + X + Width, addY, maxWidth, maxHeight
        End If
    End If

    ' Do the same with the Y-axis
    If RepeatY Then
        If addX + X + Width < maxWidth Then
            DrawPicture refSurface, addX + X + Width, addY, maxWidth, maxHeight
        End If
    End If

End Sub

Public Property Get Picture() As StdPicture
Attribute Picture.VB_Description = "Returns/sets the StdPicture."

    ' Return the picture-object
    Set Picture = pPicture

End Property

Public Property Let Picture(ByVal vNewValue As StdPicture)

    ' Set the picture to use
    Set pPicture = vNewValue
    
    ' Resize the width and height
    Width = ConvertToPixels(pPicture.Width)
    Height = ConvertToPixels(pPicture.Height)
    Clipping.Right = Width
    Clipping.Bottom = Height

End Property

Private Function ConvertToPixels(ByVal Himetric As Double) As Single
    
    ' Convert himetric to pixels
    ConvertToPixels = CLng((Himetric / 2540 * 1440) / Screen.TwipsPerPixelX)

End Function

Private Sub Class_Initialize()

    ' Set default rasteropcode
    Opcode = vbSrcCopy

End Sub

Private Sub Class_ReadProperties(PropBag As PropertyBag)

    ' Use the property
    With PropBag
    
        ' Read all properties
        X = .ReadProperty("X", 0)
        Y = .ReadProperty("Y", 0)
        Width = .ReadProperty("Width", 0)
        Height = .ReadProperty("Height", 0)
        Opcode = .ReadProperty("Opcode", vbSrcCopy)
        RepeatX = .ReadProperty("RepeatX", False)
        RepeatY = .ReadProperty("RepeatY", False)
        
        ' Load objects
        Set Clipping = .ReadProperty("Clipping", Nothing)
        Set pPicture = .ReadProperty("Picture", Nothing)
    
    End With

End Sub

Private Sub Class_WriteProperties(PropBag As PropertyBag)

    ' Use the property
    With PropBag
    
        ' Write all properties
        .WriteProperty "X", X, 0
        .WriteProperty "Y", Y, 0
        .WriteProperty "Width", Width, 0
        .WriteProperty "Height", Height, 0
        .WriteProperty "Opcode", Opcode, vbSrcCopy
        .WriteProperty "RepeatX", RepeatX, False
        .WriteProperty "RepeatY", RepeatY, False
        .WriteProperty "Clipping", Clipping, Nothing
        .WriteProperty "Picture", pPicture, Nothing

    End With

End Sub
