VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 1  'Persistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsStyle"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' This class is utilized to draw "styled" cells

' Different text-drawing flags
Enum TextFormat
    DT_WORDBREAK = &H10
    DT_VCENTER = &H4
    DT_TOP = &H0
    DT_TABSTOP = &H80
    DT_SINGLELINE = &H20
    DT_RIGHT = &H2
    DT_NOCLIP = &H100
    DT_LEFT = &H0
    DT_EXPANDTABS = &H40
    DT_INTERNAL = &H1000
    DT_EXTERNALLEADING = &H200
    DT_CENTER = &H1
    DT_BOTTOM = &H8
    DT_CALCRECT = &H400
End Enum

Enum BorderStyleTypes
    Border_Solid = 0
    Border_Dash = 1
    Border_Dot = 2
    Border_DashDot = 3
    Border_DashDotDot = 4
    Border_Transparent = 5
    Border_InsideSolid = 6
End Enum

' The name (identifyer, tag) of this style
Public Name As String
Attribute Name.VB_VarUserMemId = 0
Attribute Name.VB_VarDescription = "Returns/sets the name/key of this style."

' The color of the background square
Public BackColor As Long
Attribute BackColor.VB_VarDescription = "Returns/sets the backcolor of this style."

' The pictures to draw
Public Pictures As New Collection
Attribute Pictures.VB_VarDescription = "Returns the collection of pictures (clsPicture)."

' The color of the text
Public ForeColor As Long
Attribute ForeColor.VB_VarDescription = "Returns/sets the font color of this style."

' Internal spacing
Public Padding As New clsRect
Attribute Padding.VB_VarDescription = "Returns the inner-border of this style."

' External spacing
Public Margin As New clsRect
Attribute Margin.VB_VarDescription = "Returns the object containing the margin (outer border) of this style."

' Border-colors (should contain a collection of rects)
Public Borders As New Collection
Attribute Borders.VB_VarDescription = "Returns the collection of borders (clsRect)."

' The style and width of the borders
Public BorderStyle As BorderStyleTypes
Attribute BorderStyle.VB_VarDescription = "Returns/sets the style of the borders to be drawn."
Public BorderWidth As Long
Attribute BorderWidth.VB_VarDescription = "Returns/sets the border width."

' Text-specific flags
Public Format As TextFormat
Attribute Format.VB_VarDescription = "Returns/sets different text-drawing flags/options to use when drawing text."

' XP-theme background
Public Theme As New clsTheme
Attribute Theme.VB_VarDescription = "Returns the object representing the native XP-theme draw object."

' Whether or not this style is enabled
Public Enabled As Boolean
Attribute Enabled.VB_VarDescription = "Returns/sets whether or not this style is enabled."

' The current font
Public Font As New StdFont
Attribute Font.VB_VarDescription = "Returns/sets the current font to use."

' The main drawing procedure
Public Sub DrawCell(refSurface As Object, Dimensions As clsRect, Text As String, Optional Images As Collection, Optional Icon As String)
Attribute DrawCell.VB_Description = "Draws a cell with the current settings."

    Dim TextArea As New clsRect, Tell As Long, Picture As clsPicture
    
    ' Initialize the area to draw the text
    With TextArea
        .Left = Dimensions.Left + Padding.Left
        .Top = Dimensions.Top + Padding.Top
        .Right = Dimensions.Right - Padding.Right
        .Bottom = Dimensions.Bottom - Padding.Bottom
    End With

    ' Draw the backcolor
    If BackColor <> -1 Then
        refSurface.Line (Dimensions.Left, Dimensions.Top)-(Dimensions.Right, Dimensions.Bottom), BackColor, BF
    End If
    
    ' Draw the XP-style
    Theme.DrawTheme refSurface.hWnd, refSurface.hDC, Dimensions
    
    ' Then draw all the pictures in the style, ...
    For Each Picture In Pictures
        If (Not Picture.Icon) Or (Icon = "") Or Picture.Name = Icon Then
            Picture.DrawPicture refSurface, Dimensions.Left, Dimensions.Top, Dimensions.Right, Dimensions.Bottom
        End If
    Next
    
    ' ... as well as the given images.
    If Not (Images Is Nothing) Then
        For Each Picture In Images
            Picture.DrawPicture refSurface, Dimensions.Left, Dimensions.Top, Dimensions.Right, Dimensions.Bottom
        Next
    End If
    
    ' Set the border-styles
    refSurface.DrawStyle = BorderStyle
    refSurface.DrawWidth = BorderWidth
    
    ' Draw the four borders
    For Tell = 0 To Borders.Count - 1
        DrawLine refSurface, Dimensions.Left + Tell, Dimensions.Top + Tell, Dimensions.Left + Tell, Dimensions.Bottom - Tell, Borders(Tell + 1).Left
        DrawLine refSurface, Dimensions.Left + Tell, Dimensions.Bottom - Tell, Dimensions.Right - Tell, Dimensions.Bottom - Tell, Borders(Tell + 1).Bottom
        DrawLine refSurface, Dimensions.Right - Tell, Dimensions.Bottom - Tell, Dimensions.Right - Tell, Dimensions.Top + Tell, Borders(Tell + 1).Right
        DrawLine refSurface, Dimensions.Right - Tell, Dimensions.Top + Tell, Dimensions.Left + Tell, Dimensions.Top + Tell, Borders(Tell + 1).Top
    Next
    
    ' Reset the border style
    refSurface.DrawStyle = 0
    refSurface.DrawWidth = 1
    
    ' Swap the font itself
    Swap refSurface.Font, Font
    
    ' Use the current text color
    refSurface.ForeColor = ForeColor
    
    ' Draw the given text
    DrawText refSurface.hDC, Text, Len(Text), TextArea.UDT, Format
    
    ' Clean up
    Swap refSurface.Font, Font

End Sub

Private Sub DrawLine(refSurface As PictureBox, X1 As Long, Y1 As Long, X2 As Long, Y2 As Long, Color As Long)
    
    ' Make sure the color is valid
    If Color <> -1 Then
        refSurface.Line (X1, Y1)-(X2, Y2), Color
    End If

End Sub

Private Sub Swap(Var1 As Object, Var2 As Object)

    Dim Temp As Object
    
    ' Save the first object
    Set Temp = Var1
    
    ' Swap the two variables
    Set Var1 = Var2
    Set Var2 = Temp
    
End Sub

Public Function Clone() As clsStyle
Attribute Clone.VB_Description = "Returns a clone of this class."

    Dim Border As clsRect

    ' Firstly, create a new class
    Set Clone = New clsStyle
    
    ' Then, copy all the data
    With Clone
        .BackColor = Me.BackColor
        .ForeColor = Me.ForeColor
        .Font.Bold = Me.Font.Bold
        .Font.Charset = Me.Font.Charset
        .Font.Italic = Me.Font.Italic
        .Font.Name = Me.Font.Name
        .Font.Size = Me.Font.Size
        .Font.Strikethrough = Me.Font.Strikethrough
        .Font.Underline = Me.Font.Underline
        .Font.Weight = Me.Font.Weight
        .Format = Me.Format
        
        ' Copy rects
        Set .Margin = Me.Margin.Clone
        Set .Padding = Me.Padding.Clone
        
        ' Copy all the borders
        For Each Border In Me.Borders
            .Borders.Add Border.Clone
        Next
        
        ' Copy the theme
        Set .Theme = Theme.Clone
    
    End With

End Function

Private Sub Class_Initialize()

    ' Initialize the font
    Font.Name = "MS Sans Serif"
    Font.Size = 8
    
    ' Initialize border-width
    BorderWidth = 1
    
    ' Set defaults
    Enabled = True
    
End Sub

Private Sub Class_ReadProperties(PropBag As PropertyBag)

    Dim bEmpty() As Byte

    ' Use the given property
    With PropBag
    
        ' Read all but collections
        BackColor = .ReadProperty("Backcolor", vbWhite)
        BorderStyle = .ReadProperty("BorderStyle", 0)
        BorderWidth = .ReadProperty("BorderWidth", 1)
        Font = .ReadProperty("Font", Font)
        ForeColor = .ReadProperty("ForeColor", 0)
        Format = .ReadProperty("Format", 0)
        Name = .ReadProperty("Name", "")
        Enabled = .ReadProperty("Enabled", True)
        Set Margin = .ReadProperty("Margin", New clsRect)
        Set Padding = .ReadProperty("Padding", New clsRect)
        Set Theme = .ReadProperty("Theme", New clsTheme)

        ' Read collections
        Set Pictures = RetrieveCollection(.ReadProperty("Pictures", bEmpty))
        Set Borders = RetrieveCollection(.ReadProperty("Borders", bEmpty))
        
    End With

End Sub

Private Sub Class_WriteProperties(PropBag As PropertyBag)

    Dim oDefFont As New StdFont
    
    ' Create a default font
    With oDefFont
        .Name = "MS Sans Serif"
        .Size = 8
    End With

    ' Use the given property
    With PropBag
    
        ' Save all but collections
        .WriteProperty "BackColor", BackColor, vbWhite
        .WriteProperty "BorderStyle", BorderStyle, 0
        .WriteProperty "BorderWidth", BorderWidth, 1
        .WriteProperty "Font", Font, oDefFont
        .WriteProperty "ForeColor", ForeColor, 0
        .WriteProperty "Format", Format, 0
        .WriteProperty "Name", Name, ""
        .WriteProperty "Margin", Margin, New clsRect
        .WriteProperty "Padding", Padding, New clsRect
        .WriteProperty "Theme", Theme, New clsTheme
        .WriteProperty "Enabled", Enabled, True

        ' Save collections
        .WriteProperty "Pictures", ConvertCollection(Pictures).Contents
        .WriteProperty "Borders", ConvertCollection(Borders).Contents
        
    End With

End Sub
