Attribute VB_Name = "modDeclarations"
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Draws text within a rectangle
Declare Function DrawText Lib "user32" Alias "DrawTextA" (ByVal hDC As Long, ByVal lpStr As String, ByVal nCount As Long, lpRect As RECT, ByVal wFormat As Long) As Long

' Used to swap memory
Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)
Declare Function VarPtrArray Lib "msvbvm60.dll" Alias "VarPtr" (Var() As Any) As Long

' Basicly used to retrieve what window the mouse is currently over
Declare Function WindowFromPoint Lib "user32" (ByVal xPoint As Long, ByVal yPoint As Long) As Long
Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long

' API-calls needed for drawing XP style
Declare Function OpenThemeData Lib "uxtheme.dll" (ByVal hWnd As Long, ByVal pszClassList As Long) As Long
Declare Function CloseThemeData Lib "uxtheme.dll" (ByVal hTheme As Long) As Long
Declare Function DrawThemeParentBackground Lib "uxtheme.dll" (ByVal hWnd As Long, ByVal hDC As Long, prc As RECT) As Long
Declare Function DrawThemeBackground Lib "uxtheme.dll" (ByVal hTheme As Long, ByVal lHDC As Long, ByVal iPartId As Long, ByVal iStateId As Long, pRect As RECT, pClipRect As RECT) As Long

' Used to make a textbox numberic
Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long) As Long
Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long

Const GWL_STYLE = (-16)
Const ES_NUMBER = &H2000&

' Similar to the clsRect-class, but as a UDT
Public Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

' Used in retrieving the mouse position
Public Type POINTAPI
    x As Long
    y As Long
End Type

' The different ways of comparing values
Public Enum CompareConstants
    Compare_Equal = 0
    Compare_NotEqual = 1
    Compare_Less = 2
    Compare_More = 3
End Enum

Public Function CreateRect(Left, Top, Right, Bottom) As clsRect

    ' Firstly, create a new class
    Set CreateRect = New clsRect
    
    ' Set the data
    With CreateRect
        .Left = Left
        .Top = Top
        .Right = Right
        .Bottom = Bottom
    End With

End Function

Public Sub SwapMemory(ByVal lpDestination As Long, ByVal lpSource As Long, ByVal Size As Long)

    Dim bTemp() As Byte
    
    ' Reallocate the temporary array
    ReDim bTemp(Size - 1)
    
    ' Copy the content of the first memory into the temp array
    CopyMemory bTemp(0), ByVal lpDestination, Size
    
    ' Then swap the memory locations
    CopyMemory ByVal lpDestination, ByVal lpSource, Size
    CopyMemory ByVal lpSource, bTemp(0), Size

End Sub

Public Sub SetNumber(NumberText As TextBox, Flag As Boolean)

    Dim curStyle As Long
    
    ' Retrieve the window style
    curStyle = GetWindowLong(NumberText.hWnd, GWL_STYLE)
    
    ' Append the flag
    If Flag Then
        curStyle = curStyle Or ES_NUMBER
    Else
        curStyle = curStyle And (Not ES_NUMBER)
    End If
    
    ' Set the new style
    SetWindowLong NumberText.hWnd, GWL_STYLE, curStyle
    
    ' Refresh
    NumberText.Refresh
    
End Sub

' Used to find the index by the value of an element (only supports one-dimmension)
Public Function FindSingleElement(Items As Variant, Value As Variant, Optional Compare As CompareConstants, _
 Optional TextCompare As Boolean, Optional iStart As Variant, Optional iCountDecrease As Integer = 1, _
  Optional prItem As String = "Item", Optional prCount As String = "Count", Optional prReturn As String) As Long

    On Error Resume Next
    Dim Tell As Long, iCount As Long, vItem As Variant, vValue As Variant, bArray As Boolean
    
    ' Whether or not we're illiterating an array or an object
    bArray = ((VarType(Items) And vbArray) = vbArray)
    
    ' If the start index is not defined, attempt to find it some other way
    If IsMissing(iStart) Then
        If bArray Then
            ' We know the lower-bound when dealing with arrays.
            iStart = LBound(Items)
        Else
            ' Otherwise we're out of luck (assume 0 nevertheless)
            iStart = 0
        End If
    End If
    
    ' Firstly, retrieve the amount of elements we'll illiterate
    If bArray Then
        iCount = UBound(Items)
    Else
        iCount = CallByName(Items, prCount, VbGet)
    End If

    ' If we're using text-based comparision
    If TextCompare Then
        ' ... convert the value to lower-case
        vValue = LCase(Value)
    Else
        ' Otherwise use it as is
        vValue = Value
    End If

    ' Set the default return value
    FindSingleElement = iStart - 1

    ' Then, go through all the elements using the start- and end-position
    For Tell = iStart To iCount - iCountDecrease

        ' Retrieving the item is different if we're using an array
        If bArray Then
        
            ' Procedure return is used ONLY when comparing objects
            If prReturn = "" Then
                ' Retrieve the item as a normal data type (using the index as paramenter)
                vItem = Items(Tell)
            Else
                ' Retrieve the object
                Set vItem = Items(Tell)
            End If
        
        Else

            ' As the above
            If prReturn = "" Then
                vItem = CallByName(Items, prItem, VbGet, Tell)
            Else
                Set vItem = CallByName(Items, prItem, VbGet, Tell)
            End If
        
        End If
        
        ' See if we need to process the return value
        If prReturn <> "" Then
            ' Use the property we've set to get the value we're interested in
            vItem = CallByName(vItem, prReturn, VbGet)
        End If
        
        ' If we're comparing text, ...
        If TextCompare Then
            ' ... convert the compare value to lower-case (to avoid case-sensitivity)
            vItem = LCase(vItem)
        End If
        
        ' Compare values
        Select Case Compare
            Case Compare_Equal
                If vItem = vValue Then: FindSingleElement = Tell
            Case Compare_NotEqual
                If vItem <> vValue Then: FindSingleElement = Tell
            Case Compare_Less ' to value
                If vItem < vValue Then: FindSingleElement = Tell
            Case Compare_More ' to value
                If vItem > vValue Then: FindSingleElement = Tell
        End Select

        ' See if we're done
        If FindSingleElement <> iStart - 1 Then
            Exit For
        End If

    Next

End Function

