Attribute VB_Name = "modProperty"
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Converts a collection into a saveable property bag
Public Function ConvertCollection(refCollection As Collection) As PropertyBag

    Dim Tell As Long

    ' Create a new instance
    Set ConvertCollection = New PropertyBag

    ' Use the created bag
    With ConvertCollection

        ' Firstly, save the amount of elements
        .WriteProperty "Count", refCollection.Count
    
        ' Add all elements in the collection (*MUST* be persistable)
        For Tell = 1 To refCollection.Count
            .WriteProperty "Item" & Tell, refCollection(Tell)
        Next
    
    End With

End Function

' Converts a property bag's content into a collection
Public Function RetrieveCollection(Content() As Byte) As Collection

    On Error Resume Next
    Dim newProp As New PropertyBag, Tell As Long, oObject As Object, vKey As String
    
    ' Initialize the property bag with this content
    newProp.Contents = Content
    
    ' Then, initialize the return object
    Set RetrieveCollection = New Collection
    
    ' And add all the elements in the property bag
    For Tell = 1 To newProp.ReadProperty("Count", 0)
        
        ' Retrieve the object
        Set oObject = newProp.ReadProperty("Item" & Tell, "")
        
        ' Make sure the object exists
        If Not (oObject Is Nothing) Then
        
            ' Use the default property as key
            vKey = oObject
            
            ' Add it regardless, ...
            If LenB(vKey) = 0 Then
                ' ... but if the object dosen't have a default property, we'll ignore the key.
                RetrieveCollection.Add oObject
            Else
                ' Otherwise, use the key.
                RetrieveCollection.Add oObject, vKey
            End If
        
        End If
        
    Next

End Function

' Used in the property pages. Saves controls using the tag-member.
Public Sub SaveControls(refObject As Object, Controls As Object)

    On Error Resume Next
    Dim Control As Object, aVar As Variant, sTag As String, oObject As Object, bDone As Boolean

    ' Go through all controls
    For Each Control In Controls
        
        ' See if the tag-property is set
        If Control.Tag <> "" Then
        
            ' Split the tag into several subelements
            aVar = Split(Control.Tag, ".", 2)
        
            ' If we have to elements, retrieve the real object
            If UBound(aVar) = 1 Then
                Set oObject = CallByName(refObject, aVar(0), VbGet)
                sTag = aVar(1)
            Else
                Set oObject = refObject
                sTag = Control.Tag
            End If
        
            ' Reset variables
            bDone = False
        
            ' Get its value
            If TypeOf Control Is CheckBox Then
                CallByName oObject, sTag, VbLet, (Control.Value = 1): bDone = True
            ElseIf (TypeOf Control Is ComboBox) And Not bDone Then
                CallByName oObject, sTag, VbLet, Control.ListIndex: bDone = True
            ElseIf Not bDone Then
                CallByName oObject, sTag, VbLet, CStr(Control)
            End If
        
        End If
        
    Next

End Sub

' As the above, only here for loading the information into the controls.
Public Sub LoadControls(refObject As Object, Controls As Object)

    On Error Resume Next
    Dim Control As Object, aVar As Variant, sTag As String, oObject As Object, bDone As Boolean

    ' Go through all the controls
    For Each Control In Controls
        
        ' See if the tag-property is set
        If Control.Tag <> "" Then
        
            ' Split the tag into several subelements
            aVar = Split(Control.Tag, ".", 2)
        
            ' If we have to elements, retrieve the real object
            If UBound(aVar) = 1 Then
                Set oObject = CallByName(refObject, aVar(0), VbGet)
                sTag = aVar(1)
            Else
                Set oObject = refObject
                sTag = Control.Tag
            End If
        
            ' For some reason, the else-part of the following function got executed,
            ' even though it should have ended in the previous parts. To avoid this,
            ' I included a variable to inform the IF that it should end.
            bDone = False
            
            ' Set its value
            If TypeOf Control Is CheckBox Then
                Control.Value = IIf(CallByName(oObject, sTag, VbGet), 1, 0): bDone = True
            ElseIf (TypeOf Control Is ComboBox) And Not bDone Then
                Control.ListIndex = CallByName(oObject, sTag, VbGet): bDone = True
            ElseIf Not bDone Then
                Control = CallByName(oObject, sTag, VbGet)
            End If
        
        End If
        
    Next

End Sub
